;=====================================================
RecordVideoCaptureRT:
;=====================================================
;[ebp-16] = compatible DC
;[ebp-12] = bmp handle
;edi = screen DC
;esi = number of times to write same frame
;note: for real-time encoding, TrueFrameCount=DiffFrameCount

	mov eax,[BaseDDSurfaceADDR]
	mov ecx,[eax]
	call [EXEPROC_GETDDSURFDC]	;GetDC for DDraw surface via EXE
	mov edi,eax

	mov eax,[dibfileheader.TrueFrameCount]
	test eax,eax
	    jnz NotFirstFrameCapture
	    invoke GetTickCount
	    mov [VideoStartTime],eax
	    jmp FirstFrameCaptureOK

	NotFirstFrameCapture:
	cmp eax,[dibfileheader.MaxFrameCount]
	    jnb NoDoVideoCaptureRT

	invoke GetTickCount

	fild [dibfileheader.TrueFrameCount]
	sub esp,4
	fmul [vrdef_frametime]
	fistp DWORD [esp]
	pop ecx
	add ecx,[VideoStartTime]

	sub eax,ecx		     ;eax = time elapsed since last saved frame's time space
	    jb NoDoVideoCaptureRT

	cdq
	mov ecx,[vrdef_frametime_INT]
	div ecx 		     ;time elapsed / time per frame = # times to repeat
				     ;last frame (not current, count truncated)
	and eax,0FFh		     ;(255 max repetitions + first display)

;set to repeat previous frame if (time since last frame) > (time per frame),
;otherwise will set repeat count to 0 (copy once only)
	mov ecx,[framerepeatcount_BUF]
	mov edx,[dibfileheader.DiffFrameCount]
	mov [ecx+edx-1],al
	add [dibfileheader.TrueFrameCount],eax	      ;include repetitions in this count (add 0 if none, incremented later)
	add [dibfileheader.FrameRepCount],eax	      ;update total frame repetitions count
	mov eax,[dibfileheader.TrueFrameCount]
	sub eax,[dibfileheader.MaxFrameCount]
	    jz NoDoVideoCaptureRT	;reached exact max frame count? (will only apply if repetition)
	    jna NotOverMaxFrameCount_A	;make sure not over max frame count
	sub [ecx+edx-1],al		;update frame repeat count to stay under limit
	sub [dibfileheader.TrueFrameCount],eax
	jmp NoDoVideoCaptureRT
	NotOverMaxFrameCount_A:

	FirstFrameCaptureOK:

	invoke CreateCompatibleDC,edi
	       test eax,eax
	       mov ecx,rv_error_20
		   jz DO_REC_RT_ERROR_A

	       mov [rv_temp_dc],eax

	invoke CreateDIBSection,edi,dibfileheader.BitmapData,DIB_RGB_COLORS,DIBBitsPointer,0,0
	       test eax,eax
	       mov ecx,rv_error_21
		   jz DO_REC_RT_ERROR_A
	       mov [rv_temp_bmp_handle],eax

	invoke SelectObject,[rv_temp_dc],eax
	       mov ecx,rv_error_22
	       test eax,eax
		   jz DO_REC_RT_ERROR_A
	       cmp eax,GDI_ERROR
		   jz DO_REC_RT_ERROR_A

	invoke BitBlt,[rv_temp_dc],0,0,[dibfileheader.BitmapData.biWidth],\
			[dibfileheader.BitmapData.biHeight],edi,0,0,SRCCOPY
	       test eax,eax
	       mov ecx,rv_error_23
		   jz DO_REC_RT_ERROR_A

;write frame to file
	       push 0
	       mov edx,esp
	invoke WriteFile,[dibtempfile_handle],[DIBBitsPointer],[vrdef_bufsize],edx,0
	       pop edx
	       test eax,eax
	       mov ecx,rv_error_25
		   jz DO_REC_RT_ERROR_A
	       cmp edx,[vrdef_bufsize]
		   jnz DO_REC_RT_ERROR_A

	inc [dibfileheader.DiffFrameCount]
	inc [dibfileheader.TrueFrameCount]

	RV_FrameCaptureError_A:
	RCALL DeleteDC,[rv_temp_dc]
	RCALL DeleteObject,[rv_temp_bmp_handle] 	;also releases raw DIB data buffer

	NoDoVideoCaptureRT:

	call DisplayVideoTime

	mov eax,[BaseDDSurfaceADDR]
	push edi			 ;DDraw ReleaseDC
	mov ecx,[eax]
	call [EXEPROC_RELEASEDDSURFDC]

ret

;=====================================================
DO_REC_RT_ERROR_A:	;ecx=string
;=====================================================
	SHOWTEMPMESSAGE ecx
	mov eax,[dibfileheader.rv_rt_error_count]
	inc eax
	mov [dibfileheader.rv_rt_error_count],eax
	mov [TempMessageParam2],eax
	invoke GetLastError
	mov [TempMessageParam1],eax
jmp RV_FrameCaptureError_A

;=====================================================
DisplayVideoTime:
;=====================================================
push ebx
push ebp
mov ebp,esp
sub esp,200

	fild [dibfileheader.TrueFrameCount]
	sub esp,4
	fmul [vrdef_frametime]	;framecount*(time/frame) = elapsed time
	fistp DWORD [esp]
	pop eax

	cdq
	mov ecx,(60*60*1000)	;milliseconds/hour
	div ecx 		;get # hours passed
	mov [ebp-4],eax

	mov eax,edx		;remainder=milliseconds left
	cdq
	mov ecx,(60*1000)	;milliseconds/minute
	div ecx
	mov [ebp-8],eax

	mov eax,edx		;remainder=milliseconds left
	cdq
	mov ecx,(1000)		;milliseconds/second
	div ecx
	mov [ebp-12],eax

	lea ebx,[ebp-200]
invoke wsprintf,ebx,VideoElapsedTime_STR,[ebp-4],[ebp-8],\
		[ebp-12],[dibfileheader.TrueFrameCount],\
		[dibfileheader.MaxFrameCount],[dibfileheader.FrameRepCount]
	add esp,32
invoke TextOut,edi,0,VIDEO_TIME_MSG,ebx,eax

mov esp,ebp
pop ebp
pop ebx
ret

;=====================================================
ReleaseAllAVIBuffers:		;use in-game only
;=====================================================
;does not release AVIFIL32 dll or AVISaveOptions data
	LOCALRELEASE [framerepeatcount_BUF]
	RCALL CloseHandle,[dibtempfile_handle]
ret

;=====================================================
ReleaseAllAVIBuffersDLLExit:
;=====================================================
;must write frame repeat data to end of DIB dump here
	call ReleaseAllAVIBuffers
ret

;=====================================================
;REC cheat (un-)initialization (included in _MAIN_CHEATS.inc)
;=====================================================
Do_Video_Record_Toggle_Main:

	cmp [NoVideoCapture_BOOL],1
	    jz TOGGLENORECORDMESSAGE
	cmp [VideoEnabledBool],1
	    jz ENDVIDEOCAPTURE
	test esi,esi
	    jz NOVIDEONAMESPECIFIED

;set initial variables
	xor eax,eax
	and [VideoEnabledBool],al	 ;ensure recording disabled before starting (in case)
	mov [dibfileheader.TrueFrameCount],eax
	mov [dibfileheader.DiffFrameCount],eax
	mov [dibfileheader.FrameRepCount],eax
	mov [dibfileheader.rv_rt_error_count],eax

	fild [dibfileheader.Framerate]
	fdivr [float_1000]
	fst [vrdef_frametime]
	fistp [vrdef_frametime_INT]

	mov eax,[dibfileheader.BitmapData.biWidth]
	mov ecx,[dibfileheader.BitmapData.biHeight]
	mul ecx
	mov ecx,[vrdef_dibcolors]
	    cmp ecx,16
		jz color_depth_check_A
	    cmp ecx,24
		jz color_depth_check_A
	    cmp ecx,32
		jz color_depth_check_A
	    mov ecx,DEFAULT_VIDEO_BPP
	    mov [vrdef_dibcolors],ecx
	color_depth_check_A:
	shr ecx,3			    ;bitrate/8
	mul ecx 			    ;include dib colors in buffer size
	mov [vrdef_bufsize],eax

	mov [dibfileheader.BitmapData.biSize],BITMAPINFOHEADER
	mov [dibfileheader.BitmapData.biPlanes],1
	    mov eax,[vrdef_dibcolors]
	mov [dibfileheader.BitmapData.biBitCount],ax
	mov [dibfileheader.BitmapData.biCompression],BI_RGB	 ;direct color values

;create frame repeat count buffer (256 max repeats for every different frame)
	invoke LocalAlloc,LPTR,[dibfileheader.MaxFrameCount]
	       test eax,eax
	       mov edi,rv_error_6
		    jz CAPTURESTARTERROR
	       mov [framerepeatcount_BUF],eax

;create main DIB temp file
	       push esi
	       push edi
	       mov edi,recvideonamebuf
	       call TransferString
	       mov DWORD [edi],'.rec'
	       mov BYTE [edi+4],0
	       pop edi
	       pop esi
	invoke CreateFile,recvideonamebuf,GENERIC_READ+GENERIC_WRITE,0,0,\
		     CREATE_ALWAYS,FILE_ATTRIBUTE_TEMPORARY+FILE_FLAG_SEQUENTIAL_SCAN,0
	       cmp eax,-1
	       mov edi,rv_error_7
		    jz CAPTURESTARTERROR
	       mov [dibtempfile_handle],eax

;write initial (mainly empty or invalid) header
	invoke SetFilePointer,[dibtempfile_handle],0,0,FILE_BEGIN
	       push 0
	       mov ecx,esp
	invoke WriteFile,[dibtempfile_handle],dibfileheader,[dibfileheader.HeaderSize],ecx,0
	       pop edx
	       test eax,eax
	       mov edi,rv_error_8
		    jz CAPTURESTARTERROR
	       cmp edx,[dibfileheader.HeaderSize]
		   jnz CAPTURESTARTERROR

	or [VideoEnabledBool],1 				;start recording
jmp EXITCHEATCODE

CAPTURESTARTERROR:
	call ReleaseAllAVIBuffers
	SHOWTEMPMESSAGE edi
jmp EXITCHEATCODE

NOVIDEONAMESPECIFIED:
	SHOWTEMPMESSAGE rv_no_spec_file
jmp EXITCHEATCODE

TOGGLENORECORDMESSAGE:
	SHOWTEMPMESSAGE VideoDisabled_STR
jmp EXITCHEATCODE

ENDVIDEOCAPTURE:
;MUST SAVE FRAME REPEAT COUNT TO END OF CUSTOM DIB FILE
;AND WRITE FILE HEADER INFO AT START (SPACE PRE-ALLOCATED)

	and [VideoEnabledBool],0

;append frame repeat data to end of file
	mov eax,[vrdef_bufsize]
	mov ecx,[dibfileheader.DiffFrameCount]
	mul ecx 		;get total size of DIB data
	add eax,[dibfileheader.HeaderSize]	;eax points to EOF
	mov [dibfileheader.FrameRepeatTableFileOffset],eax
	mov [dibfileheader.FrameRepeatTableFileOffsetHigh],edx
	push edx
	mov ecx,esp
;ensure file pointer is put after the DIB data (at EOF)
	invoke SetFilePointer,[dibtempfile_handle],eax,ecx,FILE_BEGIN
	add esp,4

;write the frame repeat buffer data at end of file
	       push 0
	       mov ecx,esp
	invoke WriteFile,[dibtempfile_handle],[framerepeatcount_BUF],[dibfileheader.DiffFrameCount],ecx,0
	       mov edi,rv_error_4
	       test eax,eax
	       pop ecx
		   jz AVITerminateError_A
	       cmp ecx,[dibfileheader.DiffFrameCount]
		   jnz AVITerminateError_A

;write header information to file

	invoke SetFilePointer,[dibtempfile_handle],0,0,FILE_BEGIN

	       push 0
	       mov ecx,esp
	invoke WriteFile,[dibtempfile_handle],dibfileheader,[dibfileheader.HeaderSize],ecx,0
	       mov edi,rv_error_5
	       test eax,eax
	       pop ecx
		   jz AVITerminateError_A
	       cmp ecx,[dibfileheader.HeaderSize]
		   jnz AVITerminateError_A

		mov edi,rv_encode_success
AVITerminateError_A:
	call ReleaseAllAVIBuffers
	SHOWTEMPMESSAGE edi
jmp EXITCHEATCODE




